@@ -30,13 +30,13 @@ void F77_SUB(rrand)(double *r) { *r = unif_rand(); }
 
 void rf(double *x, int *ncol, int *nrow, int *cl, int *ncl, int *cat, 
         int *maxcat, int *addcl, int *ntree, int *nvar, int *ipi, double
-        *pi, int *nodesize, int *imp, int *iprox, int *noutlier, 
+        *pi, double *cut, int *nodesize, int *imp, int *iprox, int *noutlier, 
         double *outlier, int *outcl, int *counttr, double *prox, 
-	double *imprt, int *trace, int *ndbigtree, int *nodestatus, 
-	int *bestvar, int *treemap, int *nodeclass, double *xbestsplit, 
-	double *pid, double *errtr, int *keepf, int *testdat, double *xts,
-	int *clts, int *nts, double *countts, int *outclts, int *labelts, 
-	double *proxts, double *errts)
+        double *imprt, int *trace, int *ndbigtree, int *nodestatus, 
+        int *bestvar, int *treemap, int *nodeclass, double *xbestsplit, 
+        double *pid, double *errtr, int *keepf, int *testdat, double *xts,
+        int *clts, int *nts, double *countts, int *outclts, int *labelts, 
+        double *proxts, double *errts)
 {
   /******************************************************************
    *  C wrapper for random forests:  get input from R and drive
@@ -85,7 +85,7 @@ void rf(double *x, int *ncol, int *nrow, int *cl, int *ncl, int *cat,
     *jtr, *nc, *msum, *idmove, *jvr, *countimp,
     *at, *a, *b, *cbestsplit, *mind, *jts;
   
-  double errc;
+  double errc, maxgini = 0.0;
 
   double *tgini, *v, *tx, *wl, *classpop, *errimp,
     *rimpmarg, *tclasscat, *tclasspop, *rmargin, *win, *tp,
@@ -166,7 +166,7 @@ void rf(double *x, int *ncol, int *nrow, int *cl, int *ncl, int *cat,
 
   if(iaddcl >= 1) 
     F77_CALL(createclass)(x, cl, &nsample0, &nsample, &mdim, tdx, p,
-			  sm, ndble, &iaddcl);
+                          sm, ndble, &iaddcl);
   
   
   /*    INITIALIZE FOR RUN */
@@ -190,9 +190,9 @@ void rf(double *x, int *ncol, int *nrow, int *cl, int *ncl, int *cat,
   if(*imp == 1) {
     for(m = 0; m < mdim; ++m) {
       for(k = 0; k < nsample; ++k) {
-	for(j = 0; j < nclass; ++j) {
-	  countimp[j + (k*nclass) + (m*nclass*nsample)] = 0;
-	}
+        for(j = 0; j < nclass; ++j) {
+          countimp[j + (k*nclass) + (m*nclass*nsample)] = 0;
+        }
       }
     }
   }
@@ -200,11 +200,11 @@ void rf(double *x, int *ncol, int *nrow, int *cl, int *ncl, int *cat,
   if(*iprox == 1) {
     for(n = 0; n < nsample0; ++n)
       for(k = 0; k < nsample0; ++k)
-	prox[k + n * nsample0] = 0.0;
+         prox[k + n * nsample0] = 0.0;
     if(*testdat == 1) {
       for(n = 0; n < ntest; ++n)
-	for(k = 0; k < ntest + nsample0; ++k)
-	  proxts[n + k * ntest] = 0.0;
+        for(k = 0; k < ntest + nsample0; ++k)
+          proxts[n + k * ntest] = 0.0;
     }
   }
   
@@ -240,51 +240,52 @@ void rf(double *x, int *ncol, int *nrow, int *cl, int *ncl, int *cat,
     F77_CALL(moda)(a, &nuse, &nsample, &mdim, cat, maxcat, ncase, jin, ta);
     
     F77_CALL(buildtree)(a, b, cl, cat, &mdim, &nsample, &nclass, 
-			treemap + 2*arrayindex, 
-			bestvar + arrayindex, bestsplit, bestsplitnext, tgini,
-			nodestatus + arrayindex, nodepop, nodestart, classpop,
-			tclasspop, tclasscat, ta, &nrnodes, idmove,
-			&ndsize, ncase, parent, jin, &mtry, iv,
-			nodeclass + arrayindex, ndbigtree + jb, win, wr, wc,
-			wl, &mdim, &nuse, mind); 
+             treemap + 2*arrayindex, 
+             bestvar + arrayindex, bestsplit, bestsplitnext, tgini,
+             nodestatus + arrayindex, nodepop, nodestart, classpop,
+             tclasspop, tclasscat, ta, &nrnodes, idmove,
+             &ndsize, ncase, parent, jin, &mtry, iv,
+             nodeclass + arrayindex, ndbigtree + jb, win, wr, wc,
+             wl, &mdim, &nuse, mind); 
 
     F77_CALL(xtranslate)(x, &mdim, &nrnodes, &nsample, bestvar + arrayindex, 
-			 bestsplit, bestsplitnext, xbestsplit + arrayindex,
-			 nodestatus + arrayindex, cat, ndbigtree + jb); 
+             bestsplit, bestsplitnext, xbestsplit + arrayindex,
+             nodestatus + arrayindex, cat, ndbigtree + jb); 
 
     /*  Get test set error */
     if(*testdat == 1) {
-      F77_CALL(testreebag)(xts, &ntest, &mdim, treemap, nodestatus, 
-			   xbestsplit, cbestsplit, bestvar, nodeclass,
-			   &nrnodes, ndbigtree + jb, cat, &nclass,
-			   jts,nodexts,maxcat);
+      F77_CALL(testreebag)(xts, &ntest, &mdim, treemap + 2*arrayindex,
+			   nodestatus + arrayindex, xbestsplit + arrayindex,
+			   cbestsplit, bestvar + arrayindex, 
+			   nodeclass + arrayindex, &nrnodes, ndbigtree + jb, 
+			   cat, &nclass, jts, nodexts, maxcat);
       F77_CALL(comptserr)(countts, jts, clts, outclts, &ntest, &nclass,
-			  errts + jb, pid, labelts);
+               errts + jb, pid, labelts, cut);
     }
     
     /*  GET OUT-OF-BAG ESTIMATES */
     F77_CALL(testreebag)(x, &nsample, &mdim, treemap + 2*arrayindex, 
-			 nodestatus + arrayindex, xbestsplit + arrayindex, 
-			 cbestsplit, bestvar + arrayindex, 
-			 nodeclass + arrayindex, &nrnodes, ndbigtree + jb, 
-			 cat, &nclass, jtr, nodex, maxcat); 
+             nodestatus + arrayindex, xbestsplit + arrayindex, 
+             cbestsplit, bestvar + arrayindex, 
+             nodeclass + arrayindex, &nrnodes, ndbigtree + jb, 
+             cat, &nclass, jtr, nodex, maxcat); 
 
     for(n = 0; n < nsample; ++n) {
       if(jin[n] == 0) {
-	counttr[n*nclass + jtr[n] - 1] ++;
+        counttr[n*nclass + jtr[n] - 1] ++;
         out[n]++;
       }
     }
 
     F77_CALL(oob)(&nsample, &nclass, jin, cl, jtr, jerr, counttr, out,
-		  errtr + jb, &errc, rmargin, q, outcl, wtt);
+             errtr + jb, &errc, rmargin, q, outcl, wtt, cut);
 
     if ((jb + 1) % *trace == 0 || jb + 1 == jbt) {
       if(*trace < jbt) {
-	Rprintf("%4i: OOB error rate=%5.2f%%\t", jb+1, 100.0*errtr[jb]);
-	if(*labelts == 1) 
- 	  Rprintf("Test set error rate=%5.2f%%", 100.0*errts[jb]);
- 	Rprintf("\n");
+        Rprintf("%4i: OOB error rate=%5.2f%%\t", jb+1, 100.0*errtr[jb]);
+        if(*labelts == 1) 
+          Rprintf("Test set error rate=%5.2f%%", 100.0*errts[jb]);
+          Rprintf("\n");
       }
     }
 
@@ -293,61 +294,53 @@ void rf(double *x, int *ncol, int *nrow, int *cl, int *ncl, int *cat,
       F77_CALL(zerv)(iv, &mdim);
       for(kt = 0; kt < *(ndbigtree + jb); kt++) {
         if(nodestatus[kt + arrayindex] != -1) {
-	  iv[bestvar[kt + arrayindex] - 1] = 1;
-	  msum[bestvar[kt + arrayindex] - 1] ++;
-	}
+          iv[bestvar[kt + arrayindex] - 1] = 1;
+          msum[bestvar[kt + arrayindex] - 1] ++;
+      }
       }
       for(mr = 1; mr <= mdim; mr++) {
         if(iv[mr-1] == 1) {
-	  /* for(i = 0; i < *impiter; ++i) { */
-	  F77_CALL(permobmr)(&mr, x, tp, tx, jin, &nsample, &mdim);
-	  F77_CALL(testreebag)(x, &nsample, &mdim, treemap + 2*arrayindex, 
-			       nodestatus + arrayindex,
-			       xbestsplit + arrayindex, cbestsplit, 
-			       bestvar + arrayindex, nodeclass + arrayindex, 
-			       &nrnodes, ndbigtree + jb, cat,
-			       &nclass, jvr, nodex, maxcat); 
-	  for(n = 0; n < nsample; n++) {
-	    if(jin[n] == 0 && jtr[n] != jvr[n]) {
-	      countimp[jvr[n]-1 + n*nclass + (mr-1)*nclass*nsample]++;
-	      countimp[jtr[n]-1 + n*nclass + (mr-1)*nclass*nsample]--;
-	    }
-	  }
-	  for(n = 0; n < nsample; n++) {
-	    x[mr-1 + n*mdim] = tx[n];
-	  }
-	}
-      }/*
-      if(jb==999) {
-	for(n = 0; n < nsample; ++n) {
-	  for(mr = 0; mr < mdim; ++mr) {
-	    Rprintf("%5i ", countimp[n + mr * nsample]);
-	  }
-	  Rprintf("\n");
-	}
-	}*/
+         /* for(i = 0; i < *impiter; ++i) { */
+          F77_CALL(permobmr)(&mr, x, tp, tx, jin, &nsample, &mdim);
+          F77_CALL(testreebag)(x, &nsample, &mdim, treemap + 2*arrayindex, 
+                               nodestatus + arrayindex,
+                               xbestsplit + arrayindex, cbestsplit, 
+                               bestvar + arrayindex, nodeclass + arrayindex, 
+                               &nrnodes, ndbigtree + jb, cat,
+                               &nclass, jvr, nodex, maxcat); 
+          for(n = 0; n < nsample; n++) {
+            if(jin[n] == 0 && jtr[n] != jvr[n]) {
+              countimp[jvr[n]-1 + n*nclass + (mr-1)*nclass*nsample]++;
+              countimp[jtr[n]-1 + n*nclass + (mr-1)*nclass*nsample]--;
+            }
+          }
+          for(n = 0; n < nsample; n++) {
+            x[mr-1 + n*mdim] = tx[n];
+          }
+        }
+      }
     }
 
     /*  DO PROXIMITIES */
     if(*iprox == 1) {
       for(n = 0; n < near; ++n) {
-	for(k = 0; k < near; ++k) {
-	  if(nodex[k] == nodex[n]) prox[k*near + n] += 1.0;
-	}
+        for(k = 0; k < near; ++k) {
+          if(nodex[k] == nodex[n]) prox[k*near + n] += 1.0;
+        }
       }
       /* proximity for test data */
       if(*testdat == 1) {
-	for(n = 0; n < ntest; ++n) {
-	  for(k = 0; k <= n; ++k) {
-	    if(nodexts[k] == nodexts[n]) {
-	      proxts[k * ntest + n] += 1.0;
-	      proxts[n * ntest + k] = proxts[k * ntest + n];
-	    }
-	  }
-	  for(k = 0; k < near; ++k) {
-	    if(nodexts[n] == nodex[k]) proxts[n + ntest * (k+ntest)] += 1.0; 
-	  } 
-	}
+        for(n = 0; n < ntest; ++n) {
+          for(k = 0; k <= n; ++k) {
+            if(nodexts[k] == nodexts[n]) {
+              proxts[k * ntest + n] += 1.0;
+              proxts[n * ntest + k] = proxts[k * ntest + n];
+            }
+          }
+          for(k = 0; k < near; ++k) {
+            if(nodexts[n] == nodex[k]) proxts[n + ntest * (k+ntest)] += 1.0; 
+          } 
+        }
       } 
     }
   }
@@ -357,36 +350,43 @@ void rf(double *x, int *ncol, int *nrow, int *cl, int *ncl, int *cat,
   if(*iprox == 1) {
     for(n = 0; n < near; ++n) {
       for(k = n + 1; k < near; ++k) {
-	prox[near*k + n] /= jbt;
-	prox[near*n + k] = prox[near*k + n];
+        prox[near*k + n] /= jbt;
+        prox[near*n + k] = prox[near*k + n];
       }
       prox[near*n + n] = 1.0;
     }
     if(*testdat == 1) {
       for(n = 0; n < ntest; ++n) 
-	for(k = 0; k < ntest + nsample; ++k) 
-	  proxts[ntest*k + n] /= jbt;
+        for(k = 0; k < ntest + nsample; ++k) 
+          proxts[ntest*k + n] /= jbt;
     }
   }
 
   /*  OUTLIER DATA +++++++++++++++++++++++++++++*/
   if(*noutlier == 1) {
     F77_CALL(locateout)(prox, cl, &near, &nsample, &nclass, ncp, &iaddcl,
-			outlier, tout, isort, clp);
+             outlier, tout, isort, clp);
   }
   
   /*  IMP DATA ++++++++++++++++++++++++++++++++++*/
+  for(m = 0; m < mdim; m++) {
+    tgini[m] = tgini[m] / jbt;
+    if(tgini[m] > maxgini) maxgini = tgini[m];
+  }
   if (*imp == 1) {
     F77_CALL(finishimp)(rmissimp, countimp, out, cl, &nclass, &mdim,
-			&nsample, errimp, rimpmarg, diffmarg, cntmarg, 
-			rmargin, counttr, outcl, errtr + jbt - 1);
+                        &nsample, errimp, rimpmarg, diffmarg, cntmarg, 
+                        rmargin, counttr, outcl, errtr + jbt - 1, cut);
 
-  /*	GIVES STANDARD IMP OUTPUT */
     for(m = 0; m < mdim; m++) {
       imprt[m] = errimp[m];
       imprt[mdim + m] = ((diffmarg[m] > 0.0) ? diffmarg[m] : 0.0);
       imprt[2*mdim + m] = ((cntmarg[m] > 0.0) ? cntmarg[m] : 0.0);
-      imprt[3*mdim + m] = tgini[m] / jbt;
+      imprt[3*mdim + m] = tgini[m] / maxgini;
+    }
+  } else {
+    for(m = 0; m < mdim; ++m) {
+      imprt[m] = tgini[m] / maxgini;
     }
   }
 }
